@@ -25,11 +25,11 @@
 #include "DaemonApp.h"
 #include "DisplayManager.h"
 #include "XorgDisplayServer.h"
+#include "XorgUserDisplayServer.h"
 #include "Seat.h"
 #include "SocketServer.h"
 #include "Greeter.h"
 #include "Utils.h"
-#include "SignalHandler.h"
 
 #include <QDebug>
 #include <QFile>
@@ -38,6 +38,10 @@
 
 #include <pwd.h>
 #include <unistd.h>
+#include <sys/time.h>
+
+#include <sys/ioctl.h>
+#include <fcntl.h>
 
 #include <QDBusConnection>
 #include <QDBusMessage>
@@ -45,16 +49,89 @@
 
 #include "Login1Manager.h"
 #include "Login1Session.h"
+#include "VirtualTerminal.h"
+#include "WaylandDisplayServer.h"
+#include "config.h"
 
+static int s_ttyFailures = 0;
+#define STRINGIFY(x) #x
 
 namespace SDDM {
-    Display::Display(const int terminalId, Seat *parent) : QObject(parent),
-        m_terminalId(terminalId),
+    bool isTtyInUse(const QString &desiredTty) {
+        if (Logind::isAvailable()) {
+            OrgFreedesktopLogin1ManagerInterface manager(Logind::serviceName(), Logind::managerPath(), QDBusConnection::systemBus());
+            auto reply = manager.ListSessions();
+            reply.waitForFinished();
+
+            const auto info = reply.value();
+            for(const SessionInfo &s : info) {
+                OrgFreedesktopLogin1SessionInterface session(Logind::serviceName(), s.sessionPath.path(), QDBusConnection::systemBus());
+                if (desiredTty == session.tTY() && session.state() != QLatin1String("closing")) {
+                    qDebug() << "tty" << desiredTty << "already in use by" << session.user().path.path() << session.state()
+                                      << session.display() << session.desktop() << session.vTNr();
+                    return true;
+                }
+            }
+        }
+        return false;
+    }
+
+    int fetchAvailableVt() {
+        if (!isTtyInUse(QStringLiteral("tty" STRINGIFY(SDDM_INITIAL_VT)))) {
+            return SDDM_INITIAL_VT;
+        }
+        const auto vt = VirtualTerminal::currentVt();
+        if (vt > 0 && !isTtyInUse(QStringLiteral("tty%1").arg(vt))) {
+            return vt;
+        }
+        return VirtualTerminal::setUpNewVt();
+    }
+
+    Display::DisplayServerType Display::defaultDisplayServerType()
+    {
+        const QString &displayServerType = mainConfig.DisplayServer.get().toLower();
+        DisplayServerType ret;
+        if (displayServerType == QStringLiteral("x11-user")) {
+            ret = X11UserDisplayServerType;
+        } else if (displayServerType == QStringLiteral("wayland")) {
+            ret = WaylandDisplayServerType;
+        } else {
+            if (displayServerType != QLatin1String("x11")) {
+                qWarning("\"%s\" is an invalid value for General.DisplayServer: fall back to \"x11\"",
+                    qPrintable(displayServerType));
+            }
+            ret = X11DisplayServerType;
+        }
+        return ret;
+    }
+
+    Display::Display(Seat *parent, DisplayServerType serverType)
+        : QObject(parent),
+        m_displayServerType(serverType),
         m_auth(new Auth(this)),
-        m_displayServer(new XorgDisplayServer(this)),
         m_seat(parent),
         m_socketServer(new SocketServer(this)),
-        m_greeter(new Greeter(this)) {
+        m_greeter(new Greeter(this))
+    {
+        // Create display server
+        switch (m_displayServerType) {
+        case X11DisplayServerType:
+            m_terminalId = VirtualTerminal::setUpNewVt();
+            m_displayServer = new XorgDisplayServer(this);
+            break;
+        case X11UserDisplayServerType:
+            m_terminalId = fetchAvailableVt();
+            m_displayServer = new XorgUserDisplayServer(this);
+            m_greeter->setDisplayServerCommand(XorgUserDisplayServer::command(this));
+            break;
+        case WaylandDisplayServerType:
+            m_terminalId = fetchAvailableVt();
+            m_displayServer = new WaylandDisplayServer(this);
+            m_greeter->setDisplayServerCommand(mainConfig.Wayland.CompositorCommand.get());
+            break;
+        }
+
+        qDebug("Using VT %d", m_terminalId);
 
         // respond to authentication requests
         m_auth->setVerbose(true);
@@ -73,20 +150,40 @@ namespace SDDM {
         connect(m_socketServer, &SocketServer::login, this, &Display::login);
 
         // connect login result signals
-        connect(this, SIGNAL(loginFailed(QLocalSocket*)), m_socketServer, SLOT(loginFailed(QLocalSocket*)));
-        connect(this, SIGNAL(loginSucceeded(QLocalSocket*)), m_socketServer, SLOT(loginSucceeded(QLocalSocket*)));
+        connect(this, &Display::loginFailed, m_socketServer, &SocketServer::loginFailed);
+        connect(this, &Display::loginSucceeded, m_socketServer, &SocketServer::loginSucceeded);
+
+        connect(m_greeter, &Greeter::failed, this, &Display::stop);
+        connect(m_greeter, &Greeter::ttyFailed, this, [this] {
+            ++s_ttyFailures;
+            if (s_ttyFailures > 5) {
+                QCoreApplication::exit(23);
+            }
+            // It might be the case that we are trying a tty that has been taken over by a
+            // different process. In such a case, switch back to the initial one and try again.
+            VirtualTerminal::jumpToVt(SDDM_INITIAL_VT, true);
+            stop();
+        });
+        connect(m_greeter, &Greeter::displayServerFailed, this, &Display::displayServerFailed);
     }
 
     Display::~Display() {
+        disconnect(m_auth, &Auth::finished, this, &Display::slotHelperFinished);
         stop();
     }
 
-    QString Display::displayId() const {
-        return m_displayServer->display();
+    Display::DisplayServerType Display::displayServerType() const
+    {
+        return m_displayServerType;
     }
 
-    const int Display::terminalId() const {
-        return m_terminalId;
+    DisplayServer *Display::displayServer() const
+    {
+        return m_displayServer;
+    }
+
+    int Display::terminalId() const {
+        return m_auth->isActive() ? m_sessionTerminalId : m_terminalId;
     }
 
     const QString &Display::name() const {
@@ -154,6 +251,8 @@ namespace SDDM {
             bool success = attemptAutologin();
             if (success) {
                 return;
+            } else {
+                qWarning() << "Autologin failed!";
             }
         }
 
@@ -171,9 +270,6 @@ namespace SDDM {
             }
         }
 
-        // set greeter params
-        m_greeter->setDisplay(this);
-        m_greeter->setAuthPath(qobject_cast<XorgDisplayServer *>(m_displayServer)->authPath());
         m_greeter->setSocket(m_socketServer->socketAddress());
         m_greeter->setTheme(findGreeterTheme());
 
@@ -195,6 +291,8 @@ namespace SDDM {
         // stop the greeter
         m_greeter->stop();
 
+        m_auth->stop();
+
         // stop socket server
         m_socketServer->stop();
 
@@ -218,6 +316,7 @@ namespace SDDM {
         //the SDDM user has special privileges that skip password checking so that we can load the greeter
         //block ever trying to log in as the SDDM user
         if (user == QLatin1String("sddm")) {
+            emit loginFailed(m_socket);
             return;
         }
 
@@ -244,7 +343,8 @@ namespace SDDM {
         return QString();
     }
 
-    bool Display::findSessionEntry(const QDir &dir, const QString &name) const {
+    bool Display::findSessionEntry(const QStringList &dirPaths, const QString &name) const {
+        const QFileInfo fileInfo(name);
         QString fileName = name;
 
         // append extension
@@ -252,7 +352,17 @@ namespace SDDM {
         if (!fileName.endsWith(extension))
             fileName += extension;
 
-        return dir.exists(fileName);
+        for (const auto &path: dirPaths) {
+            QDir dir = path;
+
+            // Given an absolute path: Check that it matches dir
+            if (fileInfo.isAbsolute() && fileInfo.absolutePath() != dir.absolutePath())
+                continue;
+
+            if (dir.exists(fileName))
+                return true;
+        }
+        return false;
     }
 
     void Display::startAuth(const QString &user, const QString &password, const Session &session) {
@@ -297,54 +407,64 @@ namespace SDDM {
             }
         }
 
-        // cache last session
-        m_lastSession = session;
-
         // save session desktop file name, we'll use it to set the
         // last session later, in slotAuthenticationFinished()
         m_sessionName = session.fileName();
 
-        // some information
-        qDebug() << "Session" << m_sessionName << "selected, command:" << session.exec();
-
-        QProcessEnvironment env;
-
-        if (seat()->name() == QLatin1String("seat0")) {
-            // Use the greeter VT, for wayland sessions the helper overwrites this
-            env.insert(QStringLiteral("XDG_VTNR"), QString::number(terminalId()));
+        m_sessionTerminalId = m_terminalId;
+        if ((session.type() == Session::WaylandSession && m_displayServerType == X11DisplayServerType) || (m_greeter->isRunning() && m_displayServerType != X11DisplayServerType)) {
+            // Create a new VT when we need to have another compositor running
+            m_sessionTerminalId = VirtualTerminal::setUpNewVt();
         }
 
+        // some information
+        qDebug() << "Session" << m_sessionName << "selected, command:" << session.exec() << "for VT" << m_sessionTerminalId;
+
+        QProcessEnvironment env;
+        env.insert(session.additionalEnv());
+
         env.insert(QStringLiteral("PATH"), mainConfig.Users.DefaultPath.get());
-        if (session.xdgSessionType() == QLatin1String("x11"))
-            env.insert(QStringLiteral("DISPLAY"), name());
         env.insert(QStringLiteral("XDG_SEAT_PATH"), daemonApp->displayManager()->seatPath(seat()->name()));
         env.insert(QStringLiteral("XDG_SESSION_PATH"), daemonApp->displayManager()->sessionPath(QStringLiteral("Session%1").arg(daemonApp->newSessionId())));
         env.insert(QStringLiteral("DESKTOP_SESSION"), session.desktopSession());
-        env.insert(QStringLiteral("XDG_CURRENT_DESKTOP"), session.desktopNames());
+        if (!session.desktopNames().isEmpty())
+            env.insert(QStringLiteral("XDG_CURRENT_DESKTOP"), session.desktopNames());
         env.insert(QStringLiteral("XDG_SESSION_CLASS"), QStringLiteral("user"));
         env.insert(QStringLiteral("XDG_SESSION_TYPE"), session.xdgSessionType());
         env.insert(QStringLiteral("XDG_SEAT"), seat()->name());
+        env.insert(QStringLiteral("XDG_VTNR"), QString::number(m_sessionTerminalId));
+#ifdef HAVE_SYSTEMD
         env.insert(QStringLiteral("XDG_SESSION_DESKTOP"), session.desktopNames());
+#endif
 
-        m_auth->insertEnvironment(env);
-
+        if (session.xdgSessionType() == QLatin1String("x11")) {
+          if (m_displayServerType == X11DisplayServerType)
+            env.insert(QStringLiteral("DISPLAY"), name());
+          else
+            m_auth->setDisplayServerCommand(XorgUserDisplayServer::command(this));
+        } else {
+            m_auth->setDisplayServerCommand(QStringLiteral());
+	}
         m_auth->setUser(user);
         if (m_reuseSessionId.isNull()) {
             m_auth->setSession(session.exec());
         }
+        m_auth->insertEnvironment(env);
         m_auth->start();
     }
 
     void Display::slotAuthenticationFinished(const QString &user, bool success) {
         if (success) {
-            qDebug() << "Authenticated successfully";
+            qDebug() << "Authentication for user " << user << " successful";
 
             if (!m_reuseSessionId.isNull()) {
                 OrgFreedesktopLogin1ManagerInterface manager(Logind::serviceName(), Logind::managerPath(), QDBusConnection::systemBus());
                 manager.UnlockSession(m_reuseSessionId);
                 manager.ActivateSession(m_reuseSessionId);
+                m_started = true;
             } else {
-                m_auth->setCookie(qobject_cast<XorgDisplayServer *>(m_displayServer)->cookie());
+                if (qobject_cast<XorgDisplayServer *>(m_displayServer))
+                    m_auth->setCookie(qobject_cast<XorgDisplayServer *>(m_displayServer)->cookie());
             }
 
             // save last user and last session
@@ -361,25 +481,28 @@ namespace SDDM {
             if (m_socket)
                 emit loginSucceeded(m_socket);
         } else if (m_socket) {
-            qDebug() << "Authentication failure";
+            qDebug() << "Authentication for user " << user << " failed";
             emit loginFailed(m_socket);
         }
         m_socket = nullptr;
     }
 
     void Display::slotAuthInfo(const QString &message, Auth::Info info) {
-        // TODO: presentable to the user, eventually
-        Q_UNUSED(info);
-        qWarning() << "Authentication information:" << message;
-    }
-
-    void Display::slotAuthError(const QString &message, Auth::Error error) {
-        // TODO: handle more errors
-        qWarning() << "Authentication error:" << message;
+        qWarning() << "Authentication information:" << info << message;
 
         if (!m_socket)
             return;
 
+        m_socketServer->informationMessage(m_socket, message);
+    }
+
+    void Display::slotAuthError(const QString &message, Auth::Error error) {
+        qWarning() << "Authentication error:" << error << message;
+
+        if (!m_socket)
+            return;
+
+        m_socketServer->informationMessage(m_socket, message);
         if (error == Auth::ERROR_AUTHENTICATION)
             emit loginFailed(m_socket);
     }
@@ -406,6 +529,9 @@ namespace SDDM {
     }
 
     void Display::slotSessionStarted(bool success) {
-        qDebug() << "Session started";
+        qDebug() << "Session started" << success;
+        if (success) {
+            QTimer::singleShot(5000, m_greeter, &Greeter::stop);
+        }
     }
 }
