@@ -43,13 +43,14 @@ namespace SDDM {
             ExecRole,
             CommentRole
         };
+        Q_ENUM(SessionRole)
 
         SessionModel(QObject *parent = 0);
         ~SessionModel();
 
         QHash<int, QByteArray> roleNames() const override;
 
-        const int lastIndex() const;
+        int lastIndex() const;
 
         int rowCount(const QModelIndex &parent = QModelIndex()) const override;
         QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;
@@ -57,7 +58,7 @@ namespace SDDM {
     private:
         SessionModelPrivate *d { nullptr };
 
-        void populate(Session::Type type, const QString &path);
+        void populate(Session::Type type, const QStringList &dirPaths);
     };
 }
 
