@@ -19,6 +19,7 @@
 
 #include <QtCore/QDebug>
 #include <QtCore/QObject>
+#include <QtCore/QRegularExpression>
 
 #include "KeyboardModel.h"
 #include "KeyboardModel_p.h"
@@ -267,8 +268,7 @@ namespace SDDM {
     }
 
     QList<QString> XcbKeyboardBackend::parseShortNames(QString text) {
-        QRegExp re(QStringLiteral(R"(\+([a-z]+))"));
-        re.setCaseSensitivity(Qt::CaseInsensitive);
+        QRegularExpression re(QStringLiteral(R"(\+([a-z]+))"), QRegularExpression::CaseInsensitiveOption);
 
         QList<QString> res;
         QSet<QString> blackList; // blacklist wrong tokens
@@ -276,10 +276,11 @@ namespace SDDM {
 
         // Loop through matched substrings
         int pos = 0;
-        while ((pos = re.indexIn(text, pos)) != -1) {
-            if (!blackList.contains(re.cap(1)))
-                res << re.cap(1);
-            pos += re.matchedLength();
+        QRegularExpressionMatch match;
+        while ((match = re.match(text, pos)).hasMatch()) {
+            if (!blackList.contains(match.captured(1)))
+                res << match.captured(1);
+            pos += match.capturedLength();
         }
         return res;
     }
@@ -307,7 +308,7 @@ namespace SDDM {
     void XcbKeyboardBackend::connectEventsDispatcher(KeyboardModel *model) {
         // Setup events filter
         xcb_void_cookie_t cookie;
-        xcb_xkb_select_events_details_t foo;
+        xcb_xkb_select_events_details_t foo = {};
         xcb_generic_error_t *error = nullptr;
 
         cookie = xcb_xkb_select_events(m_conn, XCB_XKB_ID_USE_CORE_KBD,
